@@ -1,12 +1,13 @@
-import collections.abc
-from abc import ABC, abstractmethod
 from typing import Any
+from typing import Collection
+from abc import ABC
+from abc import abstractmethod
 
 
-class IterableDrop(ABC, collections.abc.Collection):
+class IterableDrop(ABC, Collection[Any]):
     """Inherit from this class if you want the drop to be iterable from
     tags like 'include' and 'render'."""
 
     @abstractmethod
-    def step(self, item: Any):
+    def step(self, item: Any) -> None:
         """Step the iterator."""
