@@ -1,4 +1,6 @@
 """Filters that don't exist in the reference implementation."""
+from __future__ import annotations
+from typing import TYPE_CHECKING
 
 try:
     from markupsafe import Markup
@@ -8,10 +10,13 @@ except ImportError:
 from liquid.filter import string_filter
 from liquid.filter import with_environment
 
+if TYPE_CHECKING:
+    from liquid import Environment
+
 
 @with_environment
 @string_filter
-def safe(val, *, environment):
+def safe(val: str, *, environment: Environment) -> str:
     """Stringify and mark as safe."""
     if environment.autoescape:
         return Markup(val)
