@@ -1,6 +1,7 @@
 """Shared configuration from which templates can be loaded and parsed."""
 
 from __future__ import annotations
+from functools import lru_cache
 from pathlib import Path
 
 from typing import Callable
@@ -10,6 +11,7 @@ from typing import Type
 from typing import Union
 from typing import Optional
 from typing import Mapping
+from typing import MutableMapping
 
 import warnings
 
@@ -76,6 +78,10 @@ class Environment:
         yield an increase in performance by avoiding calls to ``uptodate``. Defaults to
         ``True``.
     :type auto_reload: bool
+    :param cache_size: The capacity of the template cache in number of templates.
+        Defaults to 300. If ``cache_size`` is ``None`` or less than ``1``, it has the
+        effect of setting ``auto_reload`` to ``False``.
+    :type cache_size: int
     :param globals: An optional mapping that will be added to the context of any
         template loaded from this environment. Defaults to ``None``.
     :type globals: dict
@@ -95,6 +101,7 @@ class Environment:
         strict_filters: bool = True,
         autoescape: bool = False,
         auto_reload: bool = True,
+        cache_size: int = 300,
         globals: Optional[Mapping[str, object]] = None,
     ):
         self.tag_start_string = tag_start_string
@@ -125,14 +132,18 @@ class Environment:
         self.mode = tolerance
 
         # Template cache
-        self.cache = LRUCache(300)
-        self.auto_reload = auto_reload
+        if cache_size and cache_size > 0:
+            self.cache: MutableMapping[Any, Any] = LRUCache(cache_size)
+            self.auto_reload = auto_reload
+        else:
+            self.cache = {}
+            self.auto_reload = False
 
         self.template_class = BoundTemplate
 
         builtin.register(self)
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(
             (
                 self.statement_start_string,
@@ -226,7 +237,9 @@ class Environment:
 
     # pylint: disable=redefined-builtin
     def get_template(
-        self, name: str, globals: Optional[Mapping[str, object]] = None
+        self,
+        name: str,
+        globals: Optional[Mapping[str, object]] = None,
     ) -> BoundTemplate:
         """Load and parse a template using the configured loader.
 
@@ -240,18 +253,50 @@ class Environment:
             :class:`liquid.exceptions.TemplateNotFound`: if a template with the given
             name can not be found.
         """
+        template = self._check_cache(name, globals)
+
+        if not template:
+            template = self.loader.load(self, name, globals=self.make_globals(globals))
+            self.cache[name] = template
+
+        return template
+
+    async def get_template_async(
+        self,
+        name: str,
+        globals: Optional[Mapping[str, object]] = None,
+    ) -> BoundTemplate:
+        """An async version of ``get_template``."""
         template = self.cache.get(name)
 
         if isinstance(template, BoundTemplate) and (
-            not self.auto_reload or template.is_up_to_date
+            not self.auto_reload or await template.is_up_to_date_async()
         ):
             template.globals.update(self.make_globals(globals))
-            return template
+        else:
+            template = await self.loader.load_async(
+                self,
+                name,
+                globals=self.make_globals(globals),
+            )
+            self.cache[name] = template
 
-        template = self.loader.load(self, name, globals=self.make_globals(globals))
-        self.cache[name] = template
         return template
 
+    def _check_cache(
+        self,
+        name: str,
+        globals: Optional[Mapping[str, object]] = None,
+    ) -> Optional[BoundTemplate]:
+        _cached = self.cache.get(name)
+
+        if isinstance(_cached, BoundTemplate) and (
+            not self.auto_reload or _cached.is_up_to_date
+        ):
+            _cached.globals.update(self.make_globals(globals))
+            return _cached
+        return None
+
     # pylint: disable=redefined-builtin
     def make_globals(
         self, globals: Optional[Mapping[str, object]] = None
@@ -269,6 +314,7 @@ class Environment:
         msg: Optional[str] = None,
         linenum: Optional[int] = None,
     ) -> None:
+        """Raise ,warn or ignore the given exception according to the current mode."""
         if not isinstance(exc, Error):
             exc = exc(msg, linenum=linenum)
         elif not exc.linenum:
@@ -280,17 +326,10 @@ class Environment:
             warnings.warn(str(exc), category=lookup_warning(exc.__class__))
 
 
-_implicit_environments = LRUCache(10)
-
-
-def get_implicit_environment(*args):
+@lru_cache(maxsize=10)
+def get_implicit_environment(*args: Any) -> Environment:
     """Return an :class:`Environment` initialized with the given arguments."""
-    try:
-        return _implicit_environments[args]
-    except KeyError:
-        env = Environment(*args)
-        _implicit_environments[args] = env
-        return env
+    return Environment(*args)
 
 
 # `Template` is a factory function masquerading as a class. The desire to have
@@ -310,6 +349,7 @@ def Template(
     strict_filters: bool = True,
     autoescape: bool = False,
     auto_reload: bool = True,
+    cache_size: int = 300,
     globals: Optional[Mapping[str, object]] = None,
 ) -> BoundTemplate:
     """Returns a :class:`liquid.template.BoundTemplate`, automatically creating an
@@ -354,6 +394,10 @@ def Template(
         yield an increase in performance by avoiding calls to ``uptodate``. Defaults to
         ``True``.
     :type auto_reload: bool
+    :param cache_size: The capacity of the template cache in number of templates.
+        Defaults to 300. If ``cache_size`` is ``None`` or less than ``1``, it has the
+        effect of setting ``auto_reload`` to ``False``.
+    :type cache_size: int
     :param globals: An optional mapping that will be added to the context of any
         template loaded from this environment. Defaults to ``None``.
     :type globals: dict
@@ -370,6 +414,7 @@ def Template(
         strict_filters,
         autoescape,
         auto_reload,
+        cache_size,
     )
 
     return env.from_string(source, globals=globals)
