@@ -11,6 +11,8 @@ from functools import wraps
 from typing import Tuple
 from typing import Any
 from typing import Callable
+from typing import Union
+from typing import Optional
 from typing import TYPE_CHECKING
 
 from liquid.context import Undefined
@@ -22,6 +24,7 @@ if TYPE_CHECKING:
     from liquid import Environment
 
     FilterT = Callable[..., Any]
+    N = Union[float, int]
 
 
 def with_context(_filter: FilterT) -> FilterT:
@@ -51,7 +54,7 @@ def string_filter(_filter: FilterT) -> FilterT:
     string."""
 
     @wraps(_filter)
-    def wrapper(val, *args, **kwargs):
+    def wrapper(val: object, *args: Any, **kwargs: Any) -> Any:
         if not isinstance(val, str):
             val = str(val)
         try:
@@ -67,7 +70,7 @@ def array_filter(_filter: FilterT) -> FilterT:
     is not array-like."""
 
     @wraps(_filter)
-    def wrapper(val, *args, **kwargs):
+    def wrapper(val: object, *args: Any, **kwargs: Any) -> Any:
         if not isinstance(val, (list, tuple, Undefined)):
             raise FilterValueError(f"expected an array, found {type(val).__name__}")
 
@@ -83,7 +86,7 @@ def liquid_filter(_filter: FilterT) -> FilterT:
     """A filter function decorator that wraps `TypeError` in `FilterArgumentError`."""
 
     @wraps(_filter)
-    def wrapper(val, *args, **kwargs):
+    def wrapper(val: object, *args: Any, **kwargs: Any) -> Any:
         try:
             return _filter(val, *args, **kwargs)
         except TypeError as err:
@@ -92,7 +95,9 @@ def liquid_filter(_filter: FilterT) -> FilterT:
     return wrapper
 
 
-def int_arg(val, default=None):
+def int_arg(val: Any, default: Optional[int] = None) -> int:
+    """Return the ``val`` as an int or ``default`` if ``val`` can't be cast to an
+    int."""
     try:
         return int(val)
     except ValueError as err:
@@ -103,7 +108,9 @@ def int_arg(val, default=None):
         ) from err
 
 
-def num_arg(val, default=None):
+def num_arg(val: Any, default: Optional[N] = None) -> N:
+    """Return the ``val`` as an int or float. If ``val`` can't be cast to an
+    int or float, return ``default`."""
     if isinstance(val, (int, float)):
         return val
 
@@ -133,7 +140,7 @@ def math_filter(_filter: FilterT) -> FilterT:
     a number."""
 
     @wraps(_filter)
-    def wrapper(val, *args, **kwargs):
+    def wrapper(val: object, *args: Any, **kwargs: Any) -> Any:
         val = num_arg(val, default=0)
 
         try:
@@ -170,12 +177,12 @@ class Filter(ABC):
         self.env = env
 
     @abstractmethod
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args, **kwargs):  # type: ignore
         """Call the filter where `val` is the left hand expression of an output
         statement."""
 
 
-def expect_number(name: str, val: object):
+def expect_number(name: str, val: object) -> N:
     """Return `val` as an integer or float. Raise a FilterArgumentError
     if `val` can't be cast to an integer or float."""
     if isinstance(val, (int, float)):
@@ -196,7 +203,7 @@ def expect_number(name: str, val: object):
     return num
 
 
-def expect_integer(name: str, val: object):
+def expect_integer(name: str, val: object) -> int:
     """Return `val` as an integer. Raise a FilterArgumentError if `val` can't
     be cast to an integer."""
     if isinstance(val, int):
@@ -215,13 +222,13 @@ def expect_integer(name: str, val: object):
     return num
 
 
-def expect_string(name: str, val: object):
+def expect_string(name: str, val: object) -> None:
     """Raise a FilterArgumentError is val is not a string."""
     if not isinstance(val, str):
         raise FilterArgumentError(f"{name}: expected a string, found {type(val)}")
 
 
-def expect_string_or_array(name: str, val: object):
+def expect_string_or_array(name: str, val: object) -> None:
     """Raise a FilterArgumentError is val is not a string or a list."""
     if not isinstance(val, (str, list)):
         raise FilterArgumentError(
@@ -229,7 +236,7 @@ def expect_string_or_array(name: str, val: object):
         )
 
 
-def expect_n_args(name: str, n: int, args: Any, kwargs: Any):
+def expect_n_args(name: str, n: int, args: Any, kwargs: Any) -> None:
     """Raise a FilterArgumentError if args does not have `n` elements, and
     kwargs is not empty."""
     length = len(args)
@@ -243,13 +250,13 @@ def expect_n_args(name: str, n: int, args: Any, kwargs: Any):
         )
 
 
-def expect_one_arg(name: str, args: Any, kwargs: Any):
+def expect_one_arg(name: str, args: Any, kwargs: Any) -> None:
     """Raise a FilterArgumentError if args is not a single element tuple, and
     kwargs is not empty."""
     expect_n_args(name, 1, args, kwargs)
 
 
-def maybe_one_arg(name: str, args: Any, kwargs: Any):
+def maybe_one_arg(name: str, args: Any, kwargs: Any) -> bool:
     """Raise a FilterArgumentError if args is not empty or a single element tuple,
     and kwargs is not empty."""
     if kwargs:
@@ -286,7 +293,7 @@ def one_maybe_two_args(name: str, args: Any, kwargs: Any) -> Tuple[Any, ...]:
     )
 
 
-def expect_no_args(name: str, args: Any, kwargs: Any):
+def expect_no_args(name: str, args: Any, kwargs: Any) -> None:
     """Raise a FilterArgumentError if args and kwargs are not empty."""
     if args:
         raise FilterArgumentError(
@@ -298,47 +305,48 @@ def expect_no_args(name: str, args: Any, kwargs: Any):
         )
 
 
-def expect_array(name: str, val: object):
+def expect_array(name: str, val: object) -> None:
     """Raise a FilterArgumentError is val is not an array/list."""
     if not isinstance(val, (list, tuple)):
         raise FilterArgumentError(f"{name}: expected an array, found {type(val)}")
 
 
-def array_required(func):
+def array_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_array(instance.name, val)
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def array_or_string_required(func):
+def array_or_string_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_string_or_array(instance.name, val)
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def array_of_strings_required(func):
+def array_of_strings_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_array(instance.name, val)
         for item in val:
             if not isinstance(item, str):
                 raise FilterArgumentError(
-                    f"{instance.name}: expected an array of strings, found a {type(item)}"
+                    f"{instance.name}: expected an array of strings, "
+                    f"found a {type(item)}"
                 )
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def array_of_hashable_required(func):
+def array_of_hashable_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_array(instance.name, val)
         for item in val:
             if not isinstance(item, collections.abc.Hashable):
@@ -351,54 +359,54 @@ def array_of_hashable_required(func):
     return wrapper
 
 
-def string_required(func):
+def string_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_string(instance.name, val)
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def number_required(func):
+def number_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         num = expect_number(instance.name, val)
         return func(instance, num, *args, **kwargs)
 
     return wrapper
 
 
-def one_arg_required(func):
+def one_arg_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_one_arg(instance.name, args, kwargs)
         return func(instance, val, args[0], **kwargs)
 
     return wrapper
 
 
-def maybe_one_arg_required(func):
+def maybe_one_arg_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         maybe_one_arg(instance.name, args, kwargs)
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def one_maybe_two_args_required(func):
+def one_maybe_two_args_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         one_maybe_two_args(instance.name, args, kwargs)
         return func(instance, val, *args, **kwargs)
 
     return wrapper
 
 
-def one_array_arg_required(func):
+def one_array_arg_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_one_arg(instance.name, args, kwargs)
         expect_array(instance.name, args[0])
         return func(instance, val, args[0], **kwargs)
@@ -406,9 +414,9 @@ def one_array_arg_required(func):
     return wrapper
 
 
-def one_number_arg_required(func):
+def one_number_arg_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_one_arg(instance.name, args, kwargs)
         num = expect_number(instance.name, args[0])
         return func(instance, val, num, **kwargs)
@@ -416,9 +424,9 @@ def one_number_arg_required(func):
     return wrapper
 
 
-def one_string_arg_required(func):
+def one_string_arg_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_one_arg(instance.name, args, kwargs)
         expect_string(instance.name, args[0])
         return func(instance, val, args[0], **kwargs)
@@ -426,9 +434,9 @@ def one_string_arg_required(func):
     return wrapper
 
 
-def two_string_args_required(func):
+def two_string_args_required(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_n_args(instance.name, 2, args, kwargs)
         expect_string(instance.name, args[0])
         expect_string(instance.name, args[1])
@@ -437,9 +445,9 @@ def two_string_args_required(func):
     return wrapper
 
 
-def no_args(func):
+def no_args(func):  # type: ignore
     @wraps(func)
-    def wrapper(instance, val, *args, **kwargs):
+    def wrapper(instance, val, *args, **kwargs):  # type: ignore
         expect_no_args(instance.name, args, kwargs)
         return func(instance, val, *args, **kwargs)
 
