@@ -148,6 +148,48 @@ class LiquidLexerTestCase(TestCase):
                     Token(1, TOKEN_LITERAL, "</HTML>"),
                 ],
             ),
+            Case(
+                "newlines inside tags",
+                "<HTML>{%if\nproduct %}some{% else\n %}other{% endif%}</HTML>",
+                [
+                    Token(1, TOKEN_LITERAL, "<HTML>"),
+                    Token(1, TOKEN_TAG, "if"),
+                    Token(1, TOKEN_EXPRESSION, "product"),
+                    Token(2, TOKEN_LITERAL, "some"),
+                    Token(2, TOKEN_TAG, "else"),
+                    Token(3, TOKEN_LITERAL, "other"),
+                    Token(3, TOKEN_TAG, "endif"),
+                    Token(3, TOKEN_LITERAL, "</HTML>"),
+                ],
+            ),
+            Case(
+                "carriage return inside tags",
+                "<HTML>{%if\rproduct %}some{% else\r %}other{% endif%}</HTML>",
+                [
+                    Token(1, TOKEN_LITERAL, "<HTML>"),
+                    Token(1, TOKEN_TAG, "if"),
+                    Token(1, TOKEN_EXPRESSION, "product"),
+                    Token(1, TOKEN_LITERAL, "some"),
+                    Token(1, TOKEN_TAG, "else"),
+                    Token(1, TOKEN_LITERAL, "other"),
+                    Token(1, TOKEN_TAG, "endif"),
+                    Token(1, TOKEN_LITERAL, "</HTML>"),
+                ],
+            ),
+            Case(
+                "carriage return and newline inside tags",
+                "<HTML>{%if\r\nproduct %}some{% else\r\n %}other{% endif%}</HTML>",
+                [
+                    Token(1, TOKEN_LITERAL, "<HTML>"),
+                    Token(1, TOKEN_TAG, "if"),
+                    Token(1, TOKEN_EXPRESSION, "product"),
+                    Token(2, TOKEN_LITERAL, "some"),
+                    Token(2, TOKEN_TAG, "else"),
+                    Token(3, TOKEN_LITERAL, "other"),
+                    Token(3, TOKEN_TAG, "endif"),
+                    Token(3, TOKEN_LITERAL, "</HTML>"),
+                ],
+            ),
             Case(
                 "template literal and control flow, inconsistent whitespace and whitespace control",
                 "<HTML>{%if  product %}some  {%- else  %}other{% endif -%}</HTML>",
@@ -173,6 +215,17 @@ class LiquidLexerTestCase(TestCase):
                     Token(4, TOKEN_EXPRESSION, "x = 1"),
                 ],
             ),
+            Case(
+                "line numbers with carriage return",
+                ("Some\r\n\r\n{{ obj }}\r\n{% assign x = 1 %}"),
+                [
+                    Token(1, TOKEN_LITERAL, "Some\r\n\r\n"),
+                    Token(3, TOKEN_STATEMENT, "obj"),
+                    Token(3, TOKEN_LITERAL, "\r\n"),
+                    Token(4, TOKEN_TAG, "assign"),
+                    Token(4, TOKEN_EXPRESSION, "x = 1"),
+                ],
+            ),
             Case(
                 "line numbers and whitespace control",
                 "Some\n\n{{- obj -}}\n{% assign x = 1 %}",
